@@ -1,7 +1,7 @@
 """Datasource module."""
-
 from typing import Any, Dict, List, Optional, Union, cast
 
+import configparser
 import json
 import os
 from enum import Enum
@@ -17,6 +17,7 @@ from datasource_api_client.api.proxy import get_key_url, list_keys, log_metric
 from datasource_api_client.models import DatasourceConfig as APIConfig
 from datasource_api_client.models import (
     DatasourceDto,
+    DatasourceDtoAuthType,
     DatasourceDtoDataSourceType,
     ErrorResponse,
     KeyRequest,
@@ -40,6 +41,11 @@ CONFIGURATION_TYPE = "configuration"
 
 FLIGHT_ERROR_SPLIT = ". Client context:"
 
+AWS_CREDENTIALS_DEFAULT_LOCATION = "/var/lib/domino/home/.aws/credentials"
+AWS_SHARED_CREDENTIALS_FILE = "AWS_SHARED_CREDENTIALS_FILE"
+DOMINO_TOKEN_DEFAULT_LOCATION = "/var/lib/domino/home/.api/token"
+DOMINO_TOKEN_FILE = "DOMINO_TOKEN_FILE"
+
 
 class DominoError(Exception):
     """Base exception for known errors."""
@@ -50,7 +56,7 @@ class UnauthenticatedError(DominoError):
 
 
 def _unpack_flight_error(error: str) -> str:
-    """Unpack a flight error message by remove extra information."""
+    """Unpack a flight error message by removing extra information."""
     try:
         return error.split(FLIGHT_ERROR_SPLIT, maxsplit=1)[0]
     except ValueError:
@@ -60,11 +66,12 @@ def _unpack_flight_error(error: str) -> str:
 class ConfigElem(Enum):
     """Enumeration of valid config elements."""
 
-    ACCOUNT = "account"
+    ACCOUNT = "accountName"
     BUCKET = "bucket"
     DATABASE = "database"
     HOST = "host"
     PORT = "port"
+    PROJECT = "project"
     REGION = "region"
     ROLE = "role"
     SCHEMA = "schema"
@@ -74,7 +81,13 @@ class ConfigElem(Enum):
 class CredElem(Enum):
     """Enumeration of valid credential elements."""
 
+    ACCESSKEY = "accessKey"
+    ACCESSKEYID = "accessKeyID"
     PASSWORD = "password"
+    PRIVATEKEY = "privateKey"
+    SECRETACCESSKEY = "secretAccessKey"
+    SESSIONTOKEN = "sessionToken"
+    TOKEN = "token"
     USERNAME = "username"
 
 
@@ -89,7 +102,7 @@ def _cred(elem: CredElem) -> Any:
 
 def _filter_cred(att: Any, _: Any) -> Any:
     """Filter credential type attributes."""
-    return att.metadata[ELEMENT_TYPE_METADATA] == CREDENTIAL_TYPE
+    return att.metadata.get(ELEMENT_TYPE_METADATA, "") == CREDENTIAL_TYPE
 
 
 def _config(elem: ConfigElem) -> Any:
@@ -103,7 +116,7 @@ def _config(elem: ConfigElem) -> Any:
 
 def _filter_config(att: Any, _: Any) -> Any:
     """Filter configuration type attributes."""
-    return att.metadata[ELEMENT_TYPE_METADATA] == CONFIGURATION_TYPE
+    return att.metadata.get(ELEMENT_TYPE_METADATA, "") == CONFIGURATION_TYPE
 
 
 @attr.s
@@ -144,6 +157,13 @@ class ADLSConfig(Config):
     access_key: Optional[str] = _cred(elem=CredElem.PASSWORD)
 
 
+@attr.s(auto_attribs=True)
+class BigQueryConfig(Config):
+    """BigQuery datasource configuration."""
+
+    gcp_project_id: Optional[str] = _config(elem=ConfigElem.PROJECT)
+
+
 @attr.s(auto_attribs=True)
 class GCSConfig(Config):
     """GCS datasource configuration."""
@@ -174,6 +194,10 @@ class MySQLConfig(Config):
     password: Optional[str] = _cred(elem=CredElem.PASSWORD)
     username: Optional[str] = _cred(elem=CredElem.USERNAME)
 
+    aws_access_key_id: Optional[str] = _cred(elem=CredElem.ACCESSKEYID)
+    aws_secret_access_key: Optional[str] = _cred(elem=CredElem.SECRETACCESSKEY)
+    aws_session_token: Optional[str] = _cred(elem=CredElem.SESSIONTOKEN)
+
 
 @attr.s(auto_attribs=True)
 class OracleConfig(Config):
@@ -194,6 +218,10 @@ class PostgreSQLConfig(Config):
     password: Optional[str] = _cred(elem=CredElem.PASSWORD)
     username: Optional[str] = _cred(elem=CredElem.USERNAME)
 
+    aws_access_key_id: Optional[str] = _cred(elem=CredElem.ACCESSKEYID)
+    aws_secret_access_key: Optional[str] = _cred(elem=CredElem.SECRETACCESSKEY)
+    aws_session_token: Optional[str] = _cred(elem=CredElem.SESSIONTOKEN)
+
 
 @attr.s(auto_attribs=True)
 class RedshiftConfig(Config):
@@ -204,6 +232,10 @@ class RedshiftConfig(Config):
     password: Optional[str] = _cred(elem=CredElem.PASSWORD)
     username: Optional[str] = _cred(elem=CredElem.USERNAME)
 
+    aws_access_key_id: Optional[str] = _cred(elem=CredElem.ACCESSKEYID)
+    aws_secret_access_key: Optional[str] = _cred(elem=CredElem.SECRETACCESSKEY)
+    aws_session_token: Optional[str] = _cred(elem=CredElem.SESSIONTOKEN)
+
 
 @attr.s(auto_attribs=True)
 class SnowflakeConfig(Config):
@@ -216,6 +248,7 @@ class SnowflakeConfig(Config):
 
     password: Optional[str] = _cred(elem=CredElem.PASSWORD)
     username: Optional[str] = _cred(elem=CredElem.USERNAME)
+    token: Optional[str] = _cred(elem=CredElem.TOKEN)
 
 
 @attr.s(auto_attribs=True)
@@ -225,8 +258,11 @@ class S3Config(Config):
     bucket: Optional[str] = _config(elem=ConfigElem.BUCKET)
     region: Optional[str] = _config(elem=ConfigElem.REGION)
 
-    aws_access_key_id: Optional[str] = _cred(elem=CredElem.USERNAME)
-    aws_secret_access_key: Optional[str] = _cred(elem=CredElem.PASSWORD)
+    aws_access_key_id: Optional[str] = _cred(elem=CredElem.ACCESSKEYID)
+    aws_secret_access_key: Optional[str] = _cred(elem=CredElem.SECRETACCESSKEY)
+    aws_session_token: Optional[str] = _cred(elem=CredElem.SESSIONTOKEN)
+
+    profile: Optional[str] = attr.ib(default=None)
 
 
 @attr.s(auto_attribs=True)
@@ -287,10 +323,14 @@ class _Object:
     datasource: "ObjectStoreDatasource" = attr.ib(repr=False)
     key: str = attr.ib()
 
+    def http(self) -> httpx.Client:
+        """Get datasource http client."""
+        return self.datasource.http()
+
     def get(self) -> bytes:
         """Get object content as bytes."""
-        signed_url = self.datasource.get_key_url(self.key, False)
-        res = httpx.get(signed_url)
+        url = self.datasource.get_key_url(self.key, False)
+        res = self.http().get(url)
         res.raise_for_status()
 
         self.datasource.client._log_metric(  # pylint: disable=protected-access
@@ -309,9 +349,9 @@ class _Object:
         Args:
             filename: path of file to write content to.
         """
-        signed_url = self.datasource.get_key_url(self.key, False)
+        url = self.datasource.get_key_url(self.key, False)
         content_size = 0
-        with httpx.stream("GET", signed_url) as stream, open(filename, "wb") as file:
+        with self.http().stream("GET", url) as stream, open(filename, "wb") as file:
             for data in stream.iter_bytes():
                 content_size += len(data)
                 file.write(data)
@@ -329,9 +369,9 @@ class _Object:
             fileobj: A file-like object to download into.
                 At a minimum, it must implement the write method and must accept bytes.
         """
-        signed_url = self.datasource.get_key_url(self.key, False)
+        url = self.datasource.get_key_url(self.key, False)
         content_size = 0
-        with httpx.stream("GET", signed_url) as stream:
+        with self.http().stream("GET", url) as stream:
             for data in stream.iter_bytes():
                 content_size += len(data)
                 fileobj.write(data)
@@ -348,8 +388,8 @@ class _Object:
         Args:
             content: bytes content
         """
-        signed_url = self.datasource.get_key_url(self.key, True)
-        res = httpx.put(signed_url, content=content, headers=self.datasource.headers())
+        url = self.datasource.get_key_url(self.key, True)
+        res = self.http().put(url, content=content)
         res.raise_for_status()
 
         self.datasource.client._log_metric(  # pylint: disable=protected-access
@@ -364,9 +404,9 @@ class _Object:
         Args:
             filename: path of file to upload.
         """
-        signed_url = self.datasource.get_key_url(self.key, True)
+        url = self.datasource.get_key_url(self.key, True)
         with open(filename, "rb") as file:
-            res = httpx.put(signed_url, content=file, headers=self.datasource.headers())
+            res = self.http().put(url, content=file)
         res.raise_for_status()
 
         content_size = os.path.getsize(filename)
@@ -382,40 +422,130 @@ class _Object:
         Args:
             fileobj: bytes-like object or an iterable producing bytes.
         """
-        signed_url = self.datasource.get_key_url(self.key, True)
-        res = httpx.put(signed_url, content=fileobj, headers=self.datasource.headers())
+        url = self.datasource.get_key_url(self.key, True)
+        res = self.http().put(url, content=fileobj)
         res.raise_for_status()
 
 
+def load_oauth_credentials(location: str) -> Dict[str, str]:
+    """Load oauth token from given location.
+
+    Args:
+        location: location of file that contains token.
+
+    Returns:
+        .. code-block:: python
+
+            {CredElem.TOKEN.value: "token"}
+
+    Raises:
+        DominoError: if the provided location is not a valid file
+    """
+    location = location or DOMINO_TOKEN_DEFAULT_LOCATION
+    try:
+        with open(location, encoding="ascii") as token_file:
+            return {CredElem.TOKEN.value: token_file.readline().rstrip()}
+    except FileNotFoundError as exc:
+        raise DominoError("Token file location is not a valid file.") from exc
+
+
+def load_aws_credentials(location: str, profile: str = "") -> Dict[str, str]:
+    """Load AWS credentials from given location and profile.
+
+    Args:
+        location: location of file that contains token.
+        profile: profile to load.
+
+    Returns:
+        .. code-block:: python
+
+            {
+                CredElem.ACCESSKEYID.value: "access_key_id",
+                CredElem.SECRETACCESSKEY.value: "secret_access_key",
+                CredElem.SESSIONTOKEN.value: "session_token",
+            }
+
+    Raises:
+        DominoError: if the provided location is not a valid file
+    """
+    location = location or AWS_CREDENTIALS_DEFAULT_LOCATION
+    aws_config = configparser.RawConfigParser()
+    aws_config.read(location)
+    if not aws_config or not aws_config.sections():
+        raise DominoError("AWS credentials file does not exist or does not contain profiles")
+
+    profile = profile or aws_config.sections()[0]
+    return {
+        CredElem.ACCESSKEYID.value: aws_config.get(profile, "aws_access_key_id"),
+        CredElem.SECRETACCESSKEY.value: aws_config.get(profile, "aws_secret_access_key"),
+        CredElem.SESSIONTOKEN.value: aws_config.get(profile, "aws_session_token"),
+    }
+
+
 @attr.s
 class Datasource:
     """Represents a Domino datasource."""
 
     # pylint: disable=too-many-instance-attributes
 
+    auth_type: str = attr.ib()
     client: "DataSourceClient" = attr.ib(repr=False)
     config: Dict[str, Any] = attr.ib()
-    credential_type: str = attr.ib()
     datasource_type: str = attr.ib()
     identifier: str = attr.ib()
     name: str = attr.ib()
     owner: str = attr.ib()
 
     _config_override: DatasourceConfig = attr.ib(factory=Config, init=False, repr=False)
+    _httpx: Optional[httpx.Client] = attr.ib(None, init=False, repr=False)
 
     @classmethod
     def from_dto(cls, client: "DataSourceClient", dto: DatasourceDto) -> "Datasource":
         """Build a datasource from a given DTO."""
         return cls(
+            auth_type=dto.auth_type.value,
             client=client,
             config=dto.config.to_dict(),
-            credential_type=dto.credential_type.value,
             datasource_type=dto.data_source_type.value,
             identifier=dto.id,
             name=dto.name,
             owner=dto.owner_info.owner_name,
         )
 
+    def http(self) -> httpx.Client:
+        """Singleton http client built for the datasource."""
+        if self._httpx is not None:
+            return self._httpx
+
+        context = httpx.create_ssl_context()
+
+        if self.datasource_type == DatasourceDtoDataSourceType.ADLSCONFIG.value:
+            self._httpx = httpx.Client(headers=ADLS_HEADERS, verify=context)
+        elif self.datasource_type == DatasourceDtoDataSourceType.GENERICS3CONFIG.value:
+            self._httpx = httpx.Client(verify=False)
+        else:
+            self._httpx = httpx.Client(verify=context)
+        return self._httpx
+
+    def _get_credential_override(self) -> Dict[str, str]:
+        """Gets credentials override by merging service overrides and user overrides"""
+        credentials = {}
+
+        if self.auth_type == DatasourceDtoAuthType.OAUTH.value:
+            credentials = load_oauth_credentials(os.getenv(DOMINO_TOKEN_FILE, ""))
+
+        if self.auth_type in (
+            DatasourceDtoAuthType.AWSIAMROLE.value,
+            DatasourceDtoAuthType.AWSIAMROLEWITHUSERNAME.value,
+        ):
+            credentials = load_aws_credentials(
+                os.getenv(AWS_SHARED_CREDENTIALS_FILE, ""),
+                getattr(self._config_override, "profile", None),
+            )
+
+        credentials.update(self._config_override.credential())
+        return credentials
+
     def update(self, config: DatasourceConfig) -> None:
         """Store configuration override for future query calls.
 
@@ -446,7 +576,7 @@ class QueryDatasource(Datasource):
             self.identifier,
             query,
             config=self._config_override.config(),
-            credential=self._config_override.credential(),
+            credential=self._get_credential_override(),
         )
 
 
@@ -454,12 +584,6 @@ class QueryDatasource(Datasource):
 class ObjectStoreDatasource(Datasource):
     """Represents a object store type datasource."""
 
-    def headers(self) -> Dict[str, str]:
-        """Return headers for http calls to blob signed URL."""
-        if self.datasource_type == DatasourceDtoDataSourceType.ADLSCONFIG.value:
-            return ADLS_HEADERS
-        return {}
-
     def Object(self, key: str) -> _Object:  # pylint: disable=invalid-name
         """Return an object with given key and datasource client."""
         return _Object(datasource=self, key=key)
@@ -477,7 +601,7 @@ class ObjectStoreDatasource(Datasource):
             self.identifier,
             prefix,
             config=self._config_override.config(),
-            credential=self._config_override.credential(),
+            credential=self._get_credential_override(),
         )
         return [
             _Object(
@@ -505,7 +629,7 @@ class ObjectStoreDatasource(Datasource):
                 object_key,
                 is_read_write,
                 config=self._config_override.config(),
-                credential=self._config_override.credential(),
+                credential=self._get_credential_override(),
             ),
         )
 
@@ -572,6 +696,7 @@ class ObjectStoreDatasource(Datasource):
 
 DATASOURCES = {
     DatasourceDtoDataSourceType.ADLSCONFIG: ObjectStoreDatasource,
+    DatasourceDtoDataSourceType.BIGQUERYCONFIG: QueryDatasource,
     DatasourceDtoDataSourceType.GCSCONFIG: ObjectStoreDatasource,
     DatasourceDtoDataSourceType.GENERICS3CONFIG: ObjectStoreDatasource,
     DatasourceDtoDataSourceType.MYSQLCONFIG: QueryDatasource,
