@@ -47,7 +47,7 @@ _trainingset_name_pat = re.compile("^[-A-Za-z0-9_]+$")
 
 
 class ServerException(Exception):
-    """Raised when TrainingSet server rejects a request"""
+    """This exception is raised when the TrainingSet server rejects a request."""
 
     def __init__(self, message: str, server_msg: str):
         self.message = message
@@ -55,17 +55,19 @@ class ServerException(Exception):
 
 
 class SchemaMismatchException(Exception):
-    """Raised when the TrainingSet data columns do not match metadata."""
+    """This exception is raised when the TrainingSet data columns do not match the metadata."""
 
 
 def get_training_set(name: str) -> model.TrainingSet:
-    """Get a TrainingSet by name
+    """Get a TrainingSet by name.
 
     Args:
         name: Name of the training set.
 
     Returns:
-        The TrainingSet, if found."""
+        The TrainingSet, if found.
+
+    """
 
     _validate_trainingset_name(name)
 
@@ -152,7 +154,7 @@ def update_training_set(
 def delete_training_set(name: str) -> bool:
     """Delete a TrainingSet.
 
-    Will only delete if the TrainingSet has no versions.
+    **Note:** This deletes the TrainingSet only if it has no versions.
 
     Args:
         name: Name of the TrainingSet.
@@ -185,11 +187,10 @@ def create_training_set_version(
     """Create a TrainingSetVersion.
 
     Args:
-        training_set_name: Name of the TrainingSet this version belongs to.
-        df: A DataFrame holding the data.
-        training_set_name: Name of the TrainingSet this version belongs to. training_set_name must
+        training_set_name: Name of the TrainingSet this version belongs to. ``training_set_name`` must
             be a string containing only alphanumeric characters in the basic Latin alphabet
-            including dash and underscore: `[-A-Za-z_]`
+            including dash and underscore: `[-A-Za-z_]`.
+        df: A DataFrame holding the data.
         description: Description of this version.
         key_columns: Names of columns that represent IDs for retrieving features.
         target_columns: Target variables for prediction.
@@ -200,7 +201,6 @@ def create_training_set_version(
 
     Returns:
         The created TrainingSetVersion
-
     """
 
     if key_columns is None:
@@ -411,7 +411,7 @@ def list_training_set_versions(
 
 
 def _get_client() -> AuthenticatedClient:
-    domino_host = os.getenv("DOMINO_API_HOST")
+    domino_host = os.getenv("DOMINO_API_HOST", os.getenv("DOMINO_USER_HOST"))
     api_key = os.getenv("DOMINO_USER_API_KEY")
     token_file = os.getenv("DOMINO_TOKEN_FILE")
 
